# -*- coding: utf8 -*-
import os
import shutil

import Repository
import utils
import Page
import Permission


from WikiException import WikiException
class ExistPage(WikiException): pass
class ExistCategory(WikiException): pass
class NotExistCategory(WikiException): pass
class AttributeError(WikiException): pass
class InvalidCategoryName(WikiException): pass

sep = '/' 

def getCategoryByCategoryPath(categoryPath, exist=None):
	nameList = filter(None, categoryPath.split(sep))
	category = Category()
	if not nameList:
		return category
	for name in nameList:
		prev = category
		category = category.getCategory(name) # 존재하지 않는 name일땐 None을 리텀
		if not category: 
			break
	if not category and exist:
		return prev
	return category

def createCategoryByCategoryPath(categoryPath, perms=None): 
	nameList = categoryPath.split(sep)
	category = Category()
	for name in nameList:
		prev = category
		category = category.getCategory(name)
		if not category: 
			category = prev.createCategory(name, perms)  
	return category

class Category(Permission.PermissionChecker):
	def __init__(self, parent=None, categoryName=None): 
		if parent and categoryName:
			self.parent = parent
			self.categoryName = categoryName
			self.encCategoryName = utils.encode(categoryName)
			self.categoryPath = parent.joinCategoryPath(categoryName)
			self._setFilePath() # self.dirPath

		elif not (parent or categoryName): # root
			self.parent = None
			self.categoryName = ''
			self.categoryPath = ''
			self.dirPath = Repository.getPageRepository()
		else:
			raise NotExistCategory # 적당하지 않은 값으로 생성하려 했다.
		if not os.path.isdir(self.dirPath):
			raise NotExistCategory # 카테고리가 없다.
		self.loadAttr()

	def _setFilePath(self):
		self.dirPath = os.path.join(Repository.getPageRepository(), utils.encode(self.categoryPath))
	def _loadPerms(self):
		permPath = os.path.join(self.dirPath, '__Attr__')
		if not os.path.isfile(permPath):
			raise AttributeError
		s = file(permPath, 'rb').read()
		self.perms = Permission.Permission(s)
	def _editPerms(self, perms):
		file(os.path.join(self.dirPath, '__Attr__'), 'wb').write('%s\n%s' % (perms, ' '.join(self.group)))
		self._loadPerms()
	def _loadGroup(self):
		fp = file(os.path.join(self.dirPath, '__Attr__'), 'rb')
		fp.readline() # permission
		self.group = fp.readline().split()
	def _editGroup(self, uidList):
		self.group = uidList
		uids = ' '.join(self.group)
		file(os.path.join(self.dirPath, '__Attr__'), 'wb').write('%s\n%s' % (self.perms, uids))
	def editAttr(self, perms, uidList):
		self._editPerms(perms)
		self._editGroup(uidList)
	def loadAttr(self):
		self._loadPerms()
		self._loadGroup()
	def getGroup(self):
		if self.group:
			return self.group
		if self.parent:
			return self.parent.getGroup()
		return []
			
	def joinCategoryPath(self, categoryName):
		if self.categoryPath == '':
			return categoryName
		else:
			return self.categoryPath + sep + categoryName

	def getCategory(self, categoryName):
		if not categoryName.replace('/', ''):
			return None
			raise NotExistCategory# 카테고리 이름이 없음
		if not os.path.isdir(os.path.join(self.dirPath, utils.encode(categoryName))):
			return None
		return Category(self, categoryName)

	def createCategory(self, categoryName, perms=None): 
		if not utils.isValidName(categoryName):
			raise InvalidCategoryName
		childPath = os.path.join(self.dirPath, utils.encode(categoryName))
		if os.path.isdir(childPath):
			raise ExistCategory
		if os.path.isfile(childPath):
			raise ExistPage
		os.mkdir(childPath)
		if not perms:
			perms = self.perms
		file(os.path.join(childPath, '__Attr__'), 'wb').write('%s' % perms) # perms는 Permission 객체와 문자열 모두 사용 가능 
		return Category(self, categoryName)
	def remove(self):
		if not os.path.exists(self.dirPath):
			return False
		if Repository.getPageRepository() not in self.dirPath:
			return False
		for child in self.getCategoryList():
			child.remove()
		for page in self.getPageList():
			page.remove()
		if self.parent and os.path.exists(self.dirPath):
			shutil.rmtree(self.dirPath)
		return True
	def isEmpty(self):
		if not self.getPageList() and not self.getCategoryList():
			return True
		return False

	def getPage(self, pageName):
		pagePath = os.path.join(self.dirPath, utils.encode(pageName))
		if os.path.isfile(pagePath):
			return Page.Page(self, pageName)
		category = self.getCategory(pageName)
		if category:
			return category.getPage('FrontPage')
		return None

	def getPageList(self, myFilter=None):
		fileNameList = os.walk(self.dirPath).next()[2]
		fileNameList.remove('__Attr__')
		pageNameList = [utils.decode(fileName) for fileName in fileNameList]
		pageList = [Page.Page(self, pageName) for pageName in pageNameList]
		if myFilter:
			return filter(myFilter, pageList)
		return pageList 

	def getPageFromDescendants(self, pageName):
		page = self.getPage(pageName)
		if page:
			return page
		for category in self.getCategoryList():
			page = category.getPageFromDescendants(pageName)
			if page:
				return page
		return None

	def getPageListFromDescendants(self, myFilter=None):
		L = self.getPageList(myFilter)
		for category in self.getCategoryList():
			L.extend(category.getPageListFromDescendants(myFilter))
		return L

	def getCategoryList(self):
		dirNameList = os.walk(self.dirPath).next()[1]
		categoryNameList = [utils.decode(dirName) for dirName in dirNameList]
		categoryList = [Category(self, categoryName) for categoryName in categoryNameList]
		return categoryList
