# -*- coding: utf8 -*-
import os
import imLib.utils as utils
import imLib.macroProcessor
import imLib.Request as Request
import imLib.Page as Page 
import imLib.config as config

class HTMLFormatter:
	def __init__(self):
		self.debug = False
	def heading(self, title, level):
		return '<h%s><a id="%s" name="%s">%s</a></h%s>' % (level, title, title, title, level)
	def pre(self, text):
		return '<pre>%s</pre>' % text
	def tt(self, text):
		return '<tt>%s</tt>' % text
	def strong(self, text):
		return '<strong>%s</strong>' % text
	def em(self, text):
		return '<em>%s</em>' % text
	def strongEm(self, text):
		return '<strong><em>%s</em></strong>' % text
	def strike(self, text):
		return '<strike>%s</strike>' % text
	def underline(self, text):
		return '<u>%s</u>' % text
	def superscript(self, text):
		return '<sup>%s</sup>' % text
	def subscript(self, text):
		return '<sub>%s</sub>' % text
	def blockquote(self, text):
		return '<blockquote>%s</blockquote>' % text
	def q(self, text):
		return '<q>%s</q>' % text
	def horizon(self, lines):
		# level = len(lines)  두깨를 선택하게 할까?
		return '<hr />'
	def internalLink(self, categoryPath, pageName, isAbsolutePath, linkName=''):
		request = Request.Request()
		if not isAbsolutePath:
			categoryPath = utils.joinPath(request.categoryPath, categoryPath) 
		page = Page.travelAlongRails(categoryPath, pageName)
		klass = ''
		if not page:
			klass = 'notExist'
		return utils.makeLink(categoryPath, pageName, klass=klass, linkName=linkName)
	def externalLink(self, url, linkName=''):
		return '<a href="%s">%s</a>' % (url, linkName or url)

	def attachment(self, text):
		request = Request.Request()
		page = Page.getPage(request.categoryPath, request.pageName)
		if not page: # FIXME 이런 경우는 어떤경우? global page?
			return 'attachment: %s' % text
		if not page.isExistAttachment(text):
			return '<a href="%s?action=attachForm&amp;rename=%s">[upload: "%s"]</a>' % (request.actionPath, text, text) # FIXME rename기능 추가되면 이름도 넘기게..
		if os.path.splitext(text.lower())[1] in ['.jpg', '.jpeg', '.png', '.gif']: # 이걸 파서에서 해서 넘겨야하나?
			return '<img src="%s?action=download&amp;fileName=%s" alt="%s"/>' % (request.actionPath, text, text)
		return '<a href="%s?action=download&amp;fileName=%s" title="%s">Download: "%s"</a>' % (request.actionPath, text, text, text)
	def table(self, rowList):
		table = []
		table.append('<table>')
		for row in rowList:
			L = []
			for cell in row:
				if type(cell) == tuple:
					teul = '<td align="%s">%s</td>'
				else:
					teul = '<td>%s</td>'
				L.append(teul % cell)
			curRow = ''.join(L)
			table.append("<tr>%s</tr>" % curRow)
		table.append('</table>')
		return ''.join(table)


	def indent(self, parsedList): # FIXME refactoring
		L = []
		openedTypeList = []
		for i in range(len(parsedList)): 
			depth, _type, contents = parsedList[i]
			if not _type:
				tagType = 'div'
				style = ' class="imIndent"'
			elif _type == '*':
				tagType = 'ul'
				style = ''
			else:
				tagType = 'ol'
				style = ' type="%s"' % _type
			if depth > 0: 
				if _type:
					tag = '<%s%s><li>' % (tagType, style)
					openedTypeList.extend([tagType, 'li']*depth)
				else:
					tag = '<%s%s>' % (tagType, style)
					openedTypeList.extend([tagType]*depth)
				tag = tag * depth
			elif depth == 0:
				if _type:
					if parsedList[i-1][1]:
						openedTypeList.pop()
						tag = '</li><li>'
					else:
						tag = '<li>'
					openedTypeList.append('li')
				else:
					tag = ''
			elif depth < 0: 
				tag = ''	
				for j in range(abs(depth)):
					t = openedTypeList.pop()
					if t == 'li':
						tag = tag + '</li></%s>' % openedTypeList.pop()
					else:

						tag = tag + '</%s>' % t
				if _type:
					if parsedList[i-1][1]:
						openedTypeList.pop()
						tag = tag + '</li><li>'
					else:
						tag = tag + '<li>'
					openedTypeList.append('li')

			if self.debug:
				print tag
				print '%s:%s:%s' % parsedList[i]
			L.append(tag  + contents)
		tag = ''
		for tagType in openedTypeList[::-1]:
			tag = tag + '</%s>' % tagType
		L.append(tag)
		return ''.join(L)

	def listitem(self, text):
		return '<li>%s</li>' % text
	def macro(self, text):
		macroName, args = imLib.macroProcessor.extractArgs(text)
		return imLib.macroProcessor.runMacro(macroName, args)

	def smiley(self, text):
		return '<img src="%s/img/%s" alt="%s" />' % (config.urlPrefix, config.smileyDict[text], text)
	def entity(self, text):
		D = {
			'<' : '&lt;',
			'>' : '&gt;',
			'&' : '&amp;',
		}
		return D.get(text, '')
