# -*- coding: utf8 -*-
import time

import imLib.Logger as Logger
import imLib.utils as utils
import imLib.Request as Request
import imLib.Page as Page
import imLib.config as config

dateFormat = '%Y/%m/%d'
timeFormat = '[%I:%M %P]'

def makeDate(t):
	return int(time.mktime(list(time.localtime(float(t))[:3])+[0]*6))

def parse(maxItemCount=None, size=6000): # FIXME size must bigger than length of a line.
	if not maxItemCount:
		maxItemCount = config.recentChangesMaxItemCount

	request = Request.Request()
	logDict = {}
	L = []
	itemCount = 0

	lines = Logger.getRawLines(Logger.editLogFile, size)
	lines = filter(None, lines)
	lines.reverse()

	while lines and itemCount < maxItemCount:
		for line in lines:
			line = line.split('\t')
			line[1] = utils.decode(line[1])
			line[2] = utils.decode(line[2])
			categoryPath = line[1]
			pageName = line[2]
			editTime = line[3]
			pagePath = categoryPath and categoryPath+'/'+pageName or pageName
			if request.categoryPath and not pagePath.startswith(request.categoryPath+'/'):
				continue

			if pagePath not in L:
				L.append(pagePath)
				date = makeDate(editTime)
				if not logDict.has_key(date):
					logDict[date] = []
				logDict[date].append(line)
				itemCount = itemCount + 1
			if itemCount == maxItemCount:
				break
		lines = Logger.getRawLines(Logger.editLogFile, size, True)
		lines = filter(None, lines)
		lines.reverse()

	return logDict

def makeDiffLink(categoryPath, pageName): 
	page = Page.getPage(categoryPath, pageName)
	request = Request.Request()
	actionPath = request.scriptName + '/' + utils.joinPath(categoryPath, pageName)
	return '<a href="%s?action=diff"><img src="%s/img/%s.png" alt="D" /></a>' % (actionPath, config.urlPrefix, page and 'diff' or 'trashcan') 

def run(args=None):# FIXME  너무 지져분하다.
	maxItemCount = config.recentChangesMaxItemCount
	if args:
		maxItemCount = int(args[0])
	logDict = parse(maxItemCount=maxItemCount)
	if not logDict:
		return 'No Changes' #FIXME
	dateList = logDict.keys()
	myCmp = lambda lhs, rhs: cmp(rhs, lhs)
	dateList.sort(myCmp)
	L = ['<dl class="recentChanges">']	
	for date in dateList:
		L.append('<dt>%s</dt>' % time.strftime(dateFormat, time.localtime(date)))
		for logLine in logDict[date]:
			page = Page.getPage(logLine[1], logLine[2])
			pageLink = page and page.toLink() or utils.makeLink(logLine[1], logLine[2], 'notExist')
			diffLink = makeDiffLink(logLine[1], logLine[2])
			editorPage = Page.travelAlongRails('', logLine[4].strip())
			editorPageLink = editorPage and editorPage.toLink(linkName=editorPage.pageName) or logLine[4]
			L.append('''<dd>
	<span class="rcPageLink">%s %s</span>
	<span class="rcTime">%s</span>
	<span class="rcEditor">%s</span>
	<span class="rcComment">%s</span>
</dd>''' % (diffLink, pageLink, time.strftime(timeFormat, time.localtime(float(logLine[3]))), editorPageLink, utils.escape(logLine[6])))
	L.append('</dl>')
	return '\n'.join(L)
