# -*- coding: utf8 -*-
import os
import re
import cgi
import string
import imLib.Category as Category
import imLib.config as config

class IndentParser:
	def __init__(self):
		self.listMarkPat = re.compile(r'^ +(\*|(?:[a-zA-Z0-9]\.)) ')
		self.init()
		self.debug = False

	def feed(self, rawText):
		self.rawText = rawText
	def init(self):
		self.result = []
		self.indentList = []

	def parse(self):
		self.init()
		lines = self.rawText.splitlines()
		for i in range(len(lines)):
			line = lines[i]
			curIndent, _type, contents = self.split(line)
			if not contents.strip():
				if self.result:
					self.joinToPrevContents(contents)
				else: 
					self.result.append((0, None, contents+'\n'))
				continue

			if self.debug:
				print '%s;%s;%s' % self.split(line)
				print self.indentList
			if self.indentList:
				lastIndent = self.indentList[-1]
			else:
				lastIndent = 0
			indentCount = 0
			if curIndent > lastIndent:
				indentCount = 1
				self.indentList.append(curIndent)
			elif curIndent < lastIndent:
				lastIndent == self.indentList.pop()
				while self.indentList and curIndent < lastIndent:
					lastIndent = self.indentList.pop()
					indentCount = indentCount -1
				if curIndent >= lastIndent:
					self.indentList.append(lastIndent)
			elif curIndent == lastIndent and _type == None:
				self.joinToPrevContents(contents)
				continue
			self.result.append((indentCount, _type, contents+'\n'))
		return self.result
	def joinToPrevContents(self, contents):
		indentCount, _type, prevContents = self.result[-1]
		self.result[-1] = (indentCount, _type, prevContents+contents+'\n')
	def getType(self, line):
		match = self.listMarkPat.search(line)
		if match:
			return match.group(1)[0]
		return None	

	def split(self, line):
		curIndent = 0
		lineLength = len(line)
		while curIndent < lineLength and line[curIndent] == ' ':
			if self.debug:
				print 'D%s' % curIndent
			curIndent = curIndent + 1
		_type = self.getType(line)
		if _type:
			start = _type == '*' and curIndent + 2 or curIndent + 3
		else:
			start = curIndent
		return curIndent, _type, line[start:]


	def getListType(self, c): # 두번째 return은 strict로 할때를 위한..
		# Not Use.
		if c == '*':
			return 'unorder'
		elif c == 'i':
			return 'lower-roman'
		elif c == 'I':
			return 'upper-roman'
		elif c in string.lowercase:
			return 'lower-alpha'
		elif c in string.uppercase:
			return 'upper-alpha'
		elif c in string.digits:
			return 'decimal'
		return None 


class TableParser:
	def __init__(self, sep='||'):
		self.sep = sep
	def feed(self, rawText):
		self.rawText = rawText
	def parse(self):
		self.rowList = []
		for line in self.rawText.strip().splitlines():
			colList = line.strip().split(self.sep)[1:-1]
			L = []
			for col in colList:
				align, col = self.getAlign(col)
				if align:
					L.append((align, col))
				else:
					L.append(col)
			self.rowList.append(L)
		return self.rowList
	def getAlign(self, col):			
		if len(col) < 2:
			return '', col
		if col[0:2] in ['><', '<>']:
			align = 'center'
			col = col[2:]
		elif col[0] == '<':
			align = 'left'
			col = col[1:]
		elif col[0] == '>':
			align = 'right'
			col = col[1:]
		else:
			align = ''
		return align, col

def _(*args, **kw):
	ret = []
	for arg in args:
		if type(arg) in [tuple, list]:
			ret.extend(arg)
		else:
			ret.append(arg)
	for name in kw.get('remove', []):
		if name in ret:
			ret.remove(name)
	return ret

class Parser:
	textDecoration = ['strongEm', 'strong', 'em', 'strike', 'underline', 'superscript', 'subscript']
	link = ['internalLink', 'camelCase', 'relationalLink', 'externalLink', 'macro']
	etc = ['entity', 'smiley', 'attachment']
	inlineBlock = ['tt', 'inlinequote']
	inline = textDecoration + link + etc + inlineBlock
	line = ['heading', 'horizon', 'br']
	block = ['pre', 'blockquote', 'table', 'list', 'indent']

	markupMap = {
		'pre' : _(),
		'tt' : _(),
		'blockquote' : _(inline, line, block),
		'inlinequote' : _(inline),
		'table' : _(inline),
		'list' : _(inline, line, block, remove=('indent')),
		'indent' : _(inline, line, block, remove=('list', 'indent')),

		'heading' : _(inline),
		'strongEm' : _(inline),
		'strong' :  _(inline),
		'em' : _(inline),
		'strike' : _(inline),
		'underline' : _(inline),
		'superscript' : _(inline),
		'subscript' : _(inline),
		'macro' : (),
		'internalLink' : ('entity'),
		'camelCase' : ('entity'),
		'relationalLink' : ('entity'),
		'externalLink' : ('entity'),
		'entity' : (),
		'horizon' : (),
		'br' : (),
		'smiley' : (),
	}
	def __init__(self, formatter):
		self.debug = False
		self.formatter = formatter
# heading..이 제일 먼저 나와야 함. \2 그룹을 사용하기 때문에. 싫으면 (?P<heading_markup>을 따로 사용하게 고쳐줄 것.
		patterns = r"""
(?P<heading>^(={1,6}) +[^\n]+? +(\2)$[\n]?)
(^\{{3}$(?P<pre>.+?)^\}{3} *?$\n?)
(\{{3}(?P<tt>.+?)\}{3})
(^"{3}$\n(?P<blockquote>.+?)\n^"{3} *?$\n?)
("{3}(?P<inlinequote>[^"\n].+?)"{3})
(?P<table>(^(\|\|[^\n]*?)+\|\|$[\n]?)+)
(?P<indent>(^ +[^\n]+$[\n]*)+)
('{5}(?P<strongEm>[^' ].+?)'{5})
('{3}(?P<strong>[^' ].+?)'{3})
('{2}(?P<em>[^' ].+?)'{2})
(~{2}(?P<strike>[^~ ].+?)~{2})
(_{2}(?P<underline>[^_ ].+?)_{2})
(\^{2}(?P<superscript>[^\^ ].+?)\^{2})
(,{2}(?P<subscript>[^, ].+?),{2})
(\[\[(?P<macro>([^\[\n]+?))\]\])
(\[(?P<internalLink>([^\[\n]+?))\])
(?P<camelCase>/?(([A-Z][a-z]+){2,}/?)+)
(<(?P<relationalLink>([^<\n]+?))>)
(?P<externalLink>http://\S+)
((^|(?<= ))attachment:(?P<attachment>\S+))
(?P<entity>[<&>])
(?P<horizon>^-{4,}[\n]{0,1}$)
(?P<br>[\n])
(?P<smiley>%(smiley)s)
""" % {
		'smiley' : '|'.join(map(re.escape, config.smileyDict.keys())),
		}
#(@{2}(?P<attachment_short>[^@\s]+?)@{2}) #FIXME 고민중..
		patterns = patterns.strip().replace('\n', '|')
		self.rules = re.compile(patterns, re.DOTALL|re.MULTILINE)

		self.tableParser = TableParser()
		self.indentParser = IndentParser()

	def init(self):
		self.stackOfopenedMarkUp = []
		self.result = []
		self.start = 0

	def feed(self, rawText):
		self.rawText = rawText.replace('\r', '')

	def parse(self): # for next version
		self.init()
		for match in self.rules.finditer(self.rawText):
			self.result.append(self.rawText[self.start:match.start()])
			self.result.append(self.replaceAgent(match))
			self.start = match.end()
		self.result.append(self.rawText[self.start:])
		return ''.join(self.result)

	# replace methods
	def replaceAgent(self, match):
		type, matchedStr = filter(lambda group: group[1], match.groupdict().items())[0]
		if self.debug:
			print '%s : %s' % (type,repr(matchedStr))
		replFunc = getattr(self, 'repl_%s' % type)

		if type == 'relationalLink' and ('pre' in self.stackOfopenedMarkUp or 'tt' in self.stackOfopenedMarkUp):
			return cgi.escape(match.group())
		if self.stackOfopenedMarkUp and type not in self.markupMap[self.stackOfopenedMarkUp[-1]]: #FIXME 매우 급함. 매우 치명적
			if self.debug:
				print self.markupMap[self.stackOfopenedMarkUp[-1]]
			matchedStr = match.group()
			return matchedStr[0] + self.rules.sub(self.replaceAgent, matchedStr[1:]) # 임시
		self.stackOfopenedMarkUp.append(type)
		result = replFunc(matchedStr)
		self.stackOfopenedMarkUp.pop()
		return result

	def repl_indent(self, matchedStr): # FIXME
		self.indentParser.feed(matchedStr)
		parsedList = self.indentParser.parse()
		newParsedList = []
		for indentLevel, _type, contents in parsedList:
			parsedText = self.rules.sub(self.replaceAgent, contents)
			newParsedList.append((indentLevel, _type, parsedText))
		return self.formatter.indent(newParsedList)

	def repl_strong(self, matchedStr):
		return self.formatter.strong(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_em(self, matchedStr):
		return self.formatter.em(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_strongEm(self, matchedStr):
		return self.formatter.strongEm(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_blockquote(self, matchedStr):
		return self.formatter.blockquote(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_inlinequote(self, matchedStr):
		return self.formatter.q(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_strike(self, matchedStr):
		return self.formatter.strike(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_underline(self, matchedStr):
		return self.formatter.underline(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_superscript(self, matchedStr): 
		return self.formatter.superscript(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_subscript(self, matchedStr):
		return self.formatter.subscript(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_horizon(self, matchedStr): # FIXME
		return self.formatter.horizon(matchedStr)
	def repl_pre(self, matchedStr):
		return self.formatter.pre(self.rules.sub(self.replaceAgent, matchedStr))
	def repl_heading(self, matchedStr):
		level = 0
		matchedStr = matchedStr.strip()
		while matchedStr[level] == '=':
			level = level + 1
		title = matchedStr[level:-level].strip()
		return self.formatter.heading(title, level)

	def repl_table(self, matchedStr):
		self.tableParser.feed(matchedStr)
		table = self.tableParser.parse()
		newTable = []
		for row in table:
			newRow = []
			for col in row:
				if type(col) == tuple:
					newRow.append((col[0], self.rules.sub(self.replaceAgent, col[1])))
				else:
					newRow.append(self.rules.sub(self.replaceAgent, col))
			newTable.append(newRow)
		return self.formatter.table(newTable)
	def repl_tt(self, matchedStr):
		return self.formatter.tt(self.rules.sub(self.replaceAgent, matchedStr))

	def repl_internalLink(self, matchedStr, isRelationalMarkUp=False):
		if matchedStr.startswith('http://'):  #FIXME 지져분하다. 정리해라. 
			firstSpace = matchedStr.find(' ')
			if firstSpace > 0:
				url = matchedStr[:firstSpace]
				linkName = matchedStr[firstSpace+1:]
				return self.formatter.externalLink(url, linkName)
			return self.formatter.externalLink(matchedStr)
		L = matchedStr.split(Category.sep)
		pageName = L[-1]
		categoryPath = Category.sep.join(L[:-1])
		if isRelationalMarkUp:
			isAbsolutePath = matchedStr.startswith(Category.sep)
		else:
			isAbsolutePath = not matchedStr.startswith(Category.sep)
		if isAbsolutePath:
			return self.formatter.internalLink(categoryPath, pageName, isAbsolutePath)
		return self.formatter.internalLink(categoryPath, pageName, isAbsolutePath, pageName)
	def repl_camelCase(self, matchedStr):
		return self.repl_internalLink(matchedStr)
	def repl_relationalLink(self, matchedStr):
		return self.repl_internalLink(matchedStr, True)
	def repl_externalLink(self, matchedStr):
		return self.formatter.externalLink(matchedStr)
	def repl_macro(self, matchedStr):
		return self.formatter.macro(matchedStr)
	def repl_br(self, matchedStr):
		if 'pre' in self.stackOfopenedMarkUp:
			return matchedStr
		return '<br />'
	def repl_entity(self, matchedStr):
		return self.formatter.entity(matchedStr)
	def repl_smiley(self, matchedStr):
		return self.formatter.smiley(matchedStr)
	def repl_attachment(self, matchedStr):
		return self.formatter.attachment(matchedStr)

	def repl_attachment_short(self, matchedStr): # FIXME 고민중..
		return self.formatter.attachment(matchedStr)

class EchoFormatter: # FOR UNITTEST
	def __init__(self):
		self._stack = []
	def echo(self, *args):
		s = ' : '.join([str(arg) for arg in args])
		key = self._stack.pop()
		return '<%s>%s</%s>' % (key, s, key)
	def __getattr__(self, key):
		self._stack.append(key)
		return self.echo
