# -*- coding: utf8 -*-
import time
import os
import Cookie 
import sys

from utils import *
import Category
import Page


from imParser import WikiParser
from imFormatter import HTMLFormatter
import User
import UserDB
import Session
import TagManager
import Tag
import Revision
import Logger
import Request
import Response
import templateProcessor
import config
from version import *

from WikiException import WikiException
class NotExistActionError(WikiException): pass 
class InvalidRawText(WikiException): pass
class CheckYourPassword(WikiException): pass
class CheckYourEmail(WikiException): pass
class CheckYourId(WikiException): pass
class InvalidUserId(WikiException): pass

def getUserAction(actionName):
	try:
		module = __import__('imLib.imAction.' + actionName, globals(), locals(), True)
	except ImportError:
		return None
	return module.__dict__.get('do_'+actionName)

def main():
	request = Request.Request()
	request.sess = Session.Session()
	if not request.sess.has_key('uid'):
		request.sess['uid'] = 'anonymous'
	request.user = User.User(request.sess['uid'])
	templateProcessor.setTemplateFolder(config.templateFolderPath) 
	actionFunc = globals().get('do_' + request.action)
	if not actionFunc:
		actionFunc = getUserAction(request.action)
	if not actionFunc:
		raise NotExistActionError
	response = actionFunc(request)
	sys.stdout.write(str(response))

def redirect(url):
	print 'Location: %s' % url
	# for test vvvvvvvvvvvvvvvvvvvvv
	print 'Content-Type: text/plain\n'
	return 'Redirect: %s' % url 

def warning(title, warningMsg):
	request = Request.Request()
	return Response.Response('warning.teul', globals(), locals())

def do_view(request): # redirect 안쓰게-_-?
	category  = Category.getCategoryByCategoryPath(joinPath(request.categoryPath, request.pageName))
	if category:
		redirectPath = joinPath(request.scriptName, category.categoryPath, 'FrontPage')
		return redirect(redirectPath)
	if not Category.getCategoryByCategoryPath(request.categoryPath):
		return warning('Not Exist Category', 'Not exist category')

	page = Page.travelAlongRails(request.categoryPath, request.pageName)
	if not page: 
		templateCategory = Category.getCategoryByCategoryPath('Template')
		return Response.Response('noPage.teul', globals(), locals())

	#if page.category.categoryPath != 'Global' and page.category.categoryPath != request.categoryPath:
	if page.category.categoryPath != request.categoryPath and page.category.categoryPath != 'Global':
		redirectPath = joinPath(request.scriptName, page.category.categoryPath, page.pageName)
		return redirect(redirectPath)
	category = Category.getCategoryByCategoryPath(request.categoryPath) # for skin
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	parser = WikiParser.Parser(HTMLFormatter.HTMLFormatter())
	parser.feed(page.rawText)
	parsedText = parser.parse()
	tagLinks = []
	for tag in TagManager.TagManager(page.tags).toSet():
		tagLinks.append(Tag.Tag(tag).toLink())
	tagLinks = ', '.join(tagLinks)
	title = page.pageName
	return Response.Response('view.teul', globals(), locals())

def do_raw(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	try:
		rev = Revision.Revision(category, request.pageName, request.timeStamp) 
	except Revision.NotExistRevision:
		return warning('Not Exist Revision', 'Not exist revision.')
	if not rev.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	print 'Content-Type: text/plain; charset=%s\n' % config.charset
	return rev.toRawText()

def do_diff(request): 
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	page = category.getPage(request.pageName) 
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	if not request.timeStamp:
		request.timeStamp = Revision.getLastTimeStamp(request.categoryPath, request.pageName)
	if not request.timeStamp:
		request.msg = 'No Revision'
		return do_view(request)
	rev = Revision.Revision(category, request.pageName, request.timeStamp)
	ret = rev.diff(page) 
	ret = nl2br(ret) 
	title = 'Diff : %s' % page.pageName
	return Response.Response('diff.teul', globals(), locals())

def do_write(request):
	rawText = request.rawText
	tags = request.tags
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	# FIXME 1 카테고리의 권한이 없을 때 2,3카테고리를 만들어 페이지를 만들 수 있는게 아닌가.
	if not rawText.strip():
		request.msg = 'Empty text'
		return do_writeForm(request)
	category = Category.getCategoryByCategoryPath(request.writeCategoryPath)
	if not category:
		category = Category.createCategoryByCategoryPath(request.writeCategoryPath, perms)
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if category.getPage(request.pageName):
		request.msg = 'Exist page'
		return do_view(request)
	page = Page.createPage(category, request.pageName, rawText, tags, perms)
	Logger.editLog('do_write', category.categoryPath, page.pageName, time.time(), request.user.uid, request.remoteAddr, request.comment.strip())
	redirectPath = joinPath(request.scriptName, category.categoryPath, page.pageName)
	return redirect(redirectPath)

def do_writeForm(request):
	tags = ''
	rawText = ''
	templatePage = Page.getPage(config.templateCategoryName, request.template)
	if templatePage:
		tags = templatePage.tags
		rawText = templatePage.rawText
	category = Category.getCategoryByCategoryPath(request.categoryPath, exist=True) 
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = category.perms.toBoolList() # skin에서 사용
	return Response.Response('write.teul', globals(), locals())



def do_editForm(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	page = category.getPage(request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not (page.permCheck(request.user, 'read') and page.permCheck(request.user, 'write')): 
		return warning('Permission Error', "You don't have permission.")
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = page.perms.toBoolList() # skin에서 사용
	return Response.Response('edit.teul', globals(), locals())

def do_edit(request):
	rawText = request.rawText
	tags = request.tags
	mtime = request.mtime
	if not rawText.strip():
		request.msg = 'Empty text'
		return do_editForm(request)
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if mtime != str(page.mtime):
		request.msg = 'Edit conflict'
		return do_editForm(request)

	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))

	if rawText == page.rawText and tags == page.tags and perms == page.perms:
		request.msg = 'No change, No Save'
		return do_view(request)
	timeStamp = str(int(time.time()))
	page.backup(timeStamp)
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	page.edit(rawText, tags, perms) 
	Logger.editLog('do_edit', page.category.categoryPath, page.pageName, timeStamp, request.user.uid, request.remoteAddr, request.comment.strip())
	return redirect(request.actionPath)

def do_comment(request): # FIXME timeStamp를 넘겨주고 확인하는게 좋지 않을까?
#FIXME 이곳에서 수행되는 많은 동작이 다른 부분으로 넘어가거나 분리되어야 한다. webInterface에 맞지 않음.
	if not request.comment.strip():
		request.msg = 'Empty text'
		return do_view(request)
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		request.msg = 'Not exist page.'
		return do_view(request)
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	timeStamp = str(int(time.time()))
	page.backup(timeStamp)
	import re
	p = re.compile(r'{{{.+?}}}|\[\[Comment(\(.+?\))?\]\]', re.DOTALL|re.MULTILINE)
	iter = p.finditer(page.rawText)
	# 찾는다. 몇번째 Comment인지,
	# 인자를 확인한다. 어디에 추가할지.
	i = 1
	for match in iter:
		if int(request.commentMacroIndex) == i:
			break
		i = i + 1
	commentWhere = request.commentWhere
	if commentWhere == 'front': # FIXME refactoring 해야함, 아마도 간단하게 줄여질 듯.
		start = match.start()
		rawText = page.rawText[:start] + '\n' + request.comment + '\n' + page.rawText[start:]
	elif commentWhere == 'rear':
		end = match.end()
		rawText = page.rawText[:end] + '\n' + request.comment + '\n' + page.rawText[end:]
	elif commentWhere == 'top':
		rawText = request.comment + '\n' + page.rawText
	elif commentWhere == 'bottom':
		rawText = page.rawText + '\n' + request.comment 
	else:
		print 'Content-Type: text/plain\n'
		return commentWhere
		rawText = page.rawText
		pass
	page.edit(rawText, page.tags)
	Logger.editLog('do_comment', page.category.categoryPath, page.pageName, timeStamp, request.user.uid, request.remoteAddr, 'Comment added.')
	return redirect(request.actionPath)

def do_remove(request):
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		request.msg = 'Not exist page'
		return do_view(request)
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if not request.timeStamp:
		request.msg = '''<strong>Are you sure?</strong>
	<form method="POST" action="%s"> 
	<input type="hidden" name="action" value="remove">
	<input type="hidden" name="timeStamp" value="%s">
	<label for="comment">Comment</label> : <input type="text" id="comment" name="comment" size="35" />
	<input type="submit" name="button" value="Remove">
</form>''' % (request.actionPath, page.mtime)
		return do_view(request)
	if request.timeStamp != str(page.mtime):
		request.msg = 'Invalid timeStamp'
		return do_view(request)
	timeStamp = str(int(time.time()))
	page.backupAll(timeStamp)
	page.remove()
	Logger.editLog('do_remove', page.category.categoryPath, page.pageName, timeStamp, request.user.uid, request.remoteAddr, request.comment.strip())

	if os.path.exists(request.categoryPath):
		categoryPath = request.categoryPath
	else:
		categoryPath = page.category.parent and page.category.parent.categoryPath or ''
	redirectPath = joinPath(request.scriptName, categoryPath, 'RecentChanges')
	return redirect(redirectPath)

def do_titleSearch(request):
	if not request.needle.strip():
		request.msg = 'Empty needle'
		return do_view(request)
	title = "Title Search : %s" % request.needle
	page = Page.travelAlongRails(request.categoryPath, request.needle)
	if page:
		redirectPath = joinPath(request.scriptName, page.category.categoryPath, page.pageName)
		return redirect(redirectPath)
	# FIXME pageList 자식 카테고리에 여러개의 페이지가 있을경우의 pageList
	# 단어를 페이지 이름에 포함하는 pageList를 
	# 만들어줘야함.
	templateCategory = Category.getCategoryByCategoryPath('Template')
	return Response.Response('titleSearch.teul', globals(), locals())

def do_fullSearch(request):
	title = "Full Search : %s" % request.needle
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category:
		request.msg = 'Not exist category'
		return do_view(request)
	pageList = category.getPageListFromDescendants(lambda page: request.needle in page.rawText)
	return Response.Response('fullSearch.teul', globals(), locals())

def do_join(request):
	uid = request.uid.strip()
	passwd = request.passwd.strip()
	passwdConfirm = request.passwdConfirm.strip()
	email = request.email.strip()
	if not isValidName(uid, minLength=5, maxLength=16, whitespaceCheck=True) or uid.lower() == 'anonymous':
		request.msg = str(d)
		return do_view(request)
	if not (5 < len(passwd) < 17) or passwd != passwdConfirm:
		request.msg = 'Check your password (5 < 비밀번호 길이 < 17, password, passwordConfirm 이어야 합니다.)'
		return do_view(request)
	if email.count('@') != 1:
		request.msg = 'Invalid email address'
		return do_view(request)
	userDB = UserDB.UserDB()
	try:
		user = userDB.addUser(uid, passwd, email) 
	except UserDB.AlreadyExistUser:
		request.msg = 'Already exist user.'
		return do_view(request)
	request.sess['uid'] = user.uid
	return redirect(request.actionPath)

def do_login(request): 
	userDB = UserDB.UserDB()
	request.uid = request.uid.strip()
	if not request.uid:
		request.msg = 'Check your ID'
		return do_view(request)
	request.passwd = request.passwd.strip()
	if not request.passwd:
		request.msg = 'Check your password'
		return do_view(request)
	try:
		user = userDB.getUser(request.uid, request.passwd) # FIXME getUser를 써야하나? 그냥 User에서 가져오면 안되나 User.User(request.uid)
	except User.NotExistUser:
		request.msg = 'Check your ID'
		return do_view(request)
	except UserDB.NotCorrectPassword:
		request.msg = 'Check your password'
		return do_view(request)
	request.sess['uid'] = user.uid
	return redirect(request.actionPath)

def do_logout(request):
	request.sess.remove()
	request.sess['uid'] = 'anonymous'
	request.user = User.User(request.sess['uid']) 
	return redirect(request.actionPath)

def do_updateUser(request):
	# FIXME vvvv join이랑 중복
	passwd = request.passwd.strip()
	passwdConfirm = request.passwdConfirm.strip()
	email = request.email.strip()
	# uid 검사
	if request.user.uid.lower() == 'anonymous':
		request.msg = 'Invalid user ID'
		return do_view(request)
	# passwd 검사 # FIXME vvvv join이랑 중복
	if not (5 < len(passwd) < 17) or passwd != passwdConfirm:
		request.msg = 'Check your password (5 < 비밀번호 길이 < 17, password, passwordConfirm 이어야 합니다.)'
		return do_view(request)
	# email검사
	if email.count('@') != 1:
		request.msg = 'Check your email address'
		return do_view(request)
	# FIXME ^^^^^^^^^^^
	userDB = UserDB.UserDB()
	user = User.User(request.user.uid) # FIXME NotExistUser, InvalidUserData
	user.update(passwd, email)
	request.msg = 'Updated'
	return do_view(request)

def do_pageListInTag(request):
	title = "Page list in '%s' tag" % decode(request.tagName)
	tagObj = Tag.Tag(decode(request.tagName)) 
	return Response.Response('pageListInTag.teul', globals(), locals())

def do_info(request):
	timeStampList = Revision.getRevTimeStampList(request.categoryPath, request.pageName)
	timeStampList.reverse()
	if timeStampList:
		category = Category.getCategoryByCategoryPath(request.categoryPath)
		rev = Revision.Revision(category, request.pageName, timeStampList[0])  # FIXME 이 현재 권한 얻어오는게 따로 빠져야하는게 아닌가.
		if not rev.permCheck(request.user, 'read'): 
			return warning('Permission Error', "You don't have permission.")
	if not request.showAll and len(timeStampList) > 10:
		timeStampList = timeStampList[:10]
	return Response.Response('pageInfo.teul', globals(), locals())

def do_editCategoryForm(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	title = 'Category Edit : %s' % (category.categoryPath and category.categoryPath or 'Root')
	memberRead, memberWrite, groupRead, groupWrite, anonyRead, anonyWrite = category.perms.toBoolList() # skin에서 사용
	return Response.Response('editCategory.teul', globals(), locals())

def do_editCategory(request):
	category = Category.getCategoryByCategoryPath(request.categoryPath)
	perms = ''.join(map(lambda bool: bool and '1' or '0', [request.memberRead, request.memberWrite, request.groupRead, request.groupWrite, request.anonyRead, request.anonyWrite]))
	if not category.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	category.editAttr(perms, request.group.split())
	request.msg = 'changed'
	return do_editCategoryForm(request)

def do_attachForm(request):
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		return warning('Not Exist Page', "Not exist page.")
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	return Response.Response('attachment.teul', globals(), locals())

def do_attach(request):
	try:
		fileName = request.form['attachment'].filename
		fileValue = request.form['attachment'].value
	except: 
		fileName = ''
		fileValue = ''
	if not (fileName and fileValue):
		return warning('Upload Error', 'upload error')
	if request.rename:
		fileName = request.rename
		request.rename = ''
	fileName = os.path.split(fileName.replace('\\', '/'))[1]
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		return warning('Not Exist Page', "Not exist page.")
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	if fileName.startswith('.'):
		return warning('Upload Error', "You can't upload this file.")
	for ext in config.bannedExts.split(): # FIXME 음..아직 별 의미는 없지만...그냥 써둠.
		for fname in fileName.split('.')[1:]:
			if ext.lower() in fname.lower():
				return warning('Upload Error', "You can't upload %s file." % ext)
	newFileName = page.attach(fileName, fileValue)
	Logger.editLog('do_attach', page.category.categoryPath, page.pageName, time.time(), request.user.uid, request.remoteAddr, '\'%s\' is attached.' % newFileName)
	
	request.msg = "'%s' is attached." % newFileName
	return do_attachForm(request)

def do_download(request):
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		return warning('Download Error', 'Not exist page')
	if not page.permCheck(request.user, 'read'): 
		return warning('Permission Error', "You don't have permission.")
	if not request.fileName.strip():
		return warning('Download Error', 'noname...')#FIXME
	fileValue = page.getAttachment(request.fileName)
	if not fileValue:
		return warning('Download Error', 'Not exist attachment.')#FIXME
	sys.stdout.write('Content-Type: application/octet-stream\n' + 
			 'Content-Disposition: attachment; filename="%s"\n\n' % request.fileName) # FIXME RFC 2231? or RFC2047? or 그대로?
	return fileValue

def do_removeAttachment(request):
	page = Page.getPage(request.categoryPath, request.pageName)
	if not page:
		return warning('Download Error', 'Not exist page')
	if not page.permCheck(request.user, 'write'): 
		return warning('Permission Error', "You don't have permission.")
	fileName = request.fileName
	if not fileName.strip():
		return warning('Download Error', 'noname...')#FIXME
	if not request.timeStamp:
		request.msg = '''<strong>Are you sure?</strong>
	<form method="POST" action="%s"> 
	<input type="hidden" name="action" value="removeAttachment">
	<input type="hidden" name="timeStamp" value="%s">
	<input type="hidden" name="fileName" value="%s">
	<input type="submit" name="button" value="Remove">
</form>''' % (request.actionPath, page.mtime, escape(fileName))
		return do_attachForm(request)
	if request.timeStamp != str(page.mtime): # FIXME 여기선 timeStamp가 별 의미가 없군.
		request.msg = 'Invalid timeStamp'
		return do_attachForm(request)
	timeStamp = str(int(time.time()))
	newFileName = page.backupAttachment(fileName, timeStamp) # move
	Logger.editLog('do_removeAttachment', page.category.categoryPath, page.pageName, timeStamp, request.user.uid, request.remoteAddr, "'%s' is removed." % fileName)
	request.msg = "'%s' is removed." % fileName 
	return do_attachForm(request)
